local RollclearingTimer = 60	-- in sec

local locale = GetLocale()
local L = setmetatable(locale == 'zhCN' and {
	['No rolls'] = '没有掷点信息',
	['Left-Click to roll.\nRight-Click to clear the rolls.\n(Auto clear rolls after %d sec)'] = '左键掷点.\n右键清除掷点历史.\n(自动在%d秒后清除)',
} or locale == 'zhTW' and {
	['No rolls'] = '沒有擲點信息',
	['Left-Click to roll.\nRight-Click to clear the rolls.\n(Auto clear rolls after %d sec)'] = '左鍵擲點.\n右鍵清除擲點歷史.\n(自動在%d秒后清除)',
} or {}, {__index=function(t,i) return i end})

local debugf, debug = tekDebug and tekDebug:GetFrame('leafRoll'), function() end
if debugf then debug = function(...) debugf:AddMessage(strjoin(', ', ...)) end end
local addon = CreateFrame'Frame'
local rolls, sorts

local lootmethod = {
	['freeforall'] = LOOT_FREE_FOR_ALL,
	['roundrobin'] = LOOT_ROUND_ROBIN,
	['master'] = LOOT_MASTER_LOOTER,
	['group'] = LOOT_GROUP_LOOT,
	['needbeforegreed'] = LOOT_NEED_BEFORE_GREED,
}

local function GetLootMethodString()
	return (GetRealNumRaidMembers()>0 or GetRealNumPartyMembers()>0) and lootmethod[GetLootMethod()] or SOLO
end

local dataobj = LibStub('LibDataBroker-1.1'):NewDataObject('leafRoll', {
	type = 'data source',
	icon = 'Interface\\Buttons\\UI-GroupLoot-Dice-Up',
	text = '',
})

local function UpdateBrokerText()
	if rolls and sorts then
		local name = sorts[1]
		for i = 2, #sorts do
			if not sorts[i] then break end
			if rolls[sorts[i-1]] == rolls[sorts[i]] then
				name = name .. '/' .. sorts[i]
			else
				break
			end
		end
		dataobj.text = format('%s - %d', name, rolls[sorts[1]])
	else
		dataobj.text = GetLootMethodString()
	end
end
-- do it now
UpdateBrokerText()

addon:SetScript('OnEvent', function(self, event, ...) self[event](self, ...) end)
addon:RegisterEvent'CHAT_MSG_SYSTEM'
addon:RegisterEvent'RAID_ROSTER_UPDATE'
addon:RegisterEvent'PARTY_MEMBERS_CHANGED'

addon.RAID_ROSTER_UPDATE = UpdateBrokerText
addon.PARTY_MEMBERS_CHANGED = UpdateBrokerText

local total = 0
local function OnUpdate(self, el)
	total = total + el
	if total < RollclearingTimer then return end
	
	rolls = nil
	sorts = nil
	UpdateBrokerText()
	
	self:SetScript('OnUpdate', nil)
	total = 0
end

--RANDOM_ROLL_RESULT = "%s掷出%d（%d-%d）"
--RANDOM_ROLL_RESULT = "%1$s würfelt. Ergebnis: %2$d (%3$d-%4$d)"; -- Example: Jeremy rolls 4 (1-10)

local _RANDOM_ROLL_RESULT
_RANDOM_ROLL_RESULT = RANDOM_ROLL_RESULT:gsub('%(', '%%(')
_RANDOM_ROLL_RESULT = _RANDOM_ROLL_RESULT:gsub('%)', '%%)')
if locale == 'deDE' then
	_RANDOM_ROLL_RESULT = _RANDOM_ROLL_RESULT:gsub('%%1%$s', '%(%%S%+%)')
	_RANDOM_ROLL_RESULT = _RANDOM_ROLL_RESULT:gsub('%%2%$d', '%(%%d%+%)')
	_RANDOM_ROLL_RESULT = _RANDOM_ROLL_RESULT:gsub('%%3%$d', '%(%%d%+%)')
	_RANDOM_ROLL_RESULT = _RANDOM_ROLL_RESULT:gsub('%%4%$d', '%(%%d%+%)')
else
	_RANDOM_ROLL_RESULT = _RANDOM_ROLL_RESULT:gsub('%%s', '%(%%S%+%)')
	_RANDOM_ROLL_RESULT = _RANDOM_ROLL_RESULT:gsub('%%d', '%(%%d%+%)')
end

function addon:CHAT_MSG_SYSTEM(msg)
	local who, roll, minRoll, maxRoll = msg:match(_RANDOM_ROLL_RESULT)
	if not who then return end
	roll, minRoll, maxRoll = tonumber(roll), tonumber(minRoll), tonumber(maxRoll)
	debug('new roll detacted', who, roll, minRoll, maxRoll)

	if (minRoll ~= 1) or (maxRoll ~= 100) then return end
	debug('fair roll', who, roll)

	if not rolls or not sorts then
		rolls = {}
		sorts = {}
		addon:SetScript('OnUpdate', OnUpdate)
	end
	if not rolls[who] then
		rolls[who] = roll
		table.insert(sorts, who)
		debug(who, 'rolls', roll)
		
		if sorts[2] then table.sort(sorts, (function(a, b) return rolls[a] > rolls[b] end)) end
		UpdateBrokerText()
	end
end

function dataobj.OnClick(self, btn)
	if btn == 'LeftButton' then
		RandomRoll(1,100)
	else
		rolls = nil
		sorts = nil
		UpdateBrokerText()
	end
end

function dataobj.OnTooltipShow(tooltip)
	if not tooltip or not tooltip.AddLine then return end
	tooltip:AddLine('|cffff8800leafRoll|r')
	tooltip:AddLine('\n')
	
	if sorts and rolls then
		local first = rolls[sorts[1]]
		for i = 1, #sorts do
			color = rolls[sorts[i]] == first and {1,0,0,1,0,0} or {1,1,1,1,1,1}
			tooltip:AddDoubleLine(sorts[i], rolls[sorts[i]], unpack(color))
		end
	else
		tooltip:AddLine(L['No rolls'])
	end
	
	tooltip:AddLine('\n')
	tooltip:AddLine(format(L['Left-Click to roll.\nRight-Click to clear the rolls.\n(Auto clear rolls after %d sec)'], RollclearingTimer))
end
